/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#define FILENAME "/home/cascardo/fotos/debconf.jpg"
#define WIDTH 800
#define HEIGHT 600

gboolean
queue (gpointer data)
{
  gtk_widget_queue_draw (GTK_WIDGET (data));
  return TRUE;
}

gboolean
expose (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  GdkPixbuf *picture;
  GdkPixbuf *screen;
  picture = (GdkPixbuf *) data;
  screen = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
                           event->area.width, event->area.height);
  gdk_pixbuf_scale (picture, screen, 0, 0,
                    event->area.width, event->area.height,
		    -event->area.x, -event->area.y,
		    4.0, 4.0, GDK_INTERP_HYPER);
  gdk_draw_pixbuf (widget->window, NULL, screen, 0, 0, 0, 0, -1, -1,
                   GDK_RGB_DITHER_NONE, 0, 0);
  gdk_pixbuf_unref (screen);
  return FALSE;
}

int
main (int argc, char **argv)
{
  char *filename;
  GdkPixbuf *picture;
  GdkColorspace colorspace;
  gboolean has_alpha;
  int bits_per_sample;
  int width, height;
  GtkWidget *window;
  GtkWidget *draw;
  gtk_init (&argc, &argv);
  if (argc < 2)
    filename = FILENAME;
  else
    filename = argv[1];
  picture = gdk_pixbuf_new_from_file (filename, NULL);
  colorspace = gdk_pixbuf_get_colorspace (picture);
  has_alpha = gdk_pixbuf_get_has_alpha (picture);
  bits_per_sample = gdk_pixbuf_get_bits_per_sample (picture);
  width = WIDTH;
  height = HEIGHT;
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect (G_OBJECT (window), "destroy",
                    G_CALLBACK (gtk_main_quit), NULL);
  draw = gtk_drawing_area_new ();
  gtk_widget_set_size_request (draw, width, height);
  gtk_container_add (GTK_CONTAINER (window), draw);
  gtk_widget_show_all (window);
  g_signal_connect (G_OBJECT (draw), "expose_event",
                    G_CALLBACK (expose), picture);
  g_timeout_add (10, queue, draw);
  gtk_main ();
  gdk_pixbuf_unref (picture);
  return 0;
}
