/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _GNU_SOURCE
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "point.h"

#define SWAP(x, y) do { \
	x ^= y; y ^= x; x ^= y; \
	} while (0)

void
InsertLine (GArray *points, Point *src, Point *dst)
{
  Point rect;
  int inc, err, thre, swap;
  int x1, y1, x2, y2;
  int x, y;
  rect.name = src->name;
  err = 0;
  swap = 0;
  x1 = src->x;
  y1 = src->y;
  x2 = dst->x;
  y2 = dst->y;
  inc = y2 - y1;
  thre = x2 - x1;
  if (ABS (inc) > ABS (thre))
  {
    SWAP (inc, thre);
    SWAP (x1, y1);
    SWAP (x2, y2);
    swap = 1;
  }
  for (y = y1, x = x1; (x2 < x1) ? (x >= x2) : (x <= x2); (x2 < x1) ? x-- : x++)
  {
    rect.x = (swap ? y : x);
    rect.y = (swap ? x : y);
    g_array_append_val (points, rect);
    err += ABS (inc);
    if (err >= ABS (thre))
    {
      err -= ABS (thre);
      y += (inc < 0) ? -1 : 1;
    }
  }
}

GArray *
ReadPoints (char *filename)
{
  GArray *points;
  FILE *file;
  char *buffer;
  char *next;
  size_t len;
  ssize_t r;
  int i = 0;
  Point last;
  Point rect;
  file = fopen (filename, "r");
  points = g_array_new (FALSE, TRUE, sizeof (Point));
  buffer = NULL;
  len = 0;
  rect.x = rect.y = 0;
  rect.name = NULL;
  last = rect;
  while (!feof (file))
  {
    r = getline (&buffer, &len, file);
    buffer[r - 1] = '\0';
    rect.x = strtol (buffer, &next, 0);
    rect.y = strtol (next+1, &next, 0);
    strtol (next, &next, 0);
    if (i > 0)
      InsertLine (points, &last, &rect);
    while (isspace (*next)) next++;
    rect.name = g_strdup (next);
    last = rect;
    i++;
  }
  fclose (file);
  return points;
}
